@@ -62,10 +62,12 @@ from langchain_aws.utils import create_aws_client
 logger = logging.getLogger(__name__)
 _BM = TypeVar("_BM", bound=BaseModel)
 
+EMPTY_CONTENT = "."
+
 MIME_TO_FORMAT = {
     # Image formats
     "image/png": "png",
-    "image/jpeg": "jpeg", 
+    "image/jpeg": "jpeg",
     "image/gif": "gif",
     "image/webp": "webp",
     # File formats
@@ -533,9 +535,11 @@ class ChatBedrockConverse(BaseChatModel):
         return values
 
     @classmethod
-    def _get_streaming_support(cls, provider: str, model_id_lower: str) -> Union[bool, str]:
+    def _get_streaming_support(
+        cls, provider: str, model_id_lower: str
+    ) -> Union[bool, str]:
         """Determine streaming support for a given provider and model.
-        
+
         Returns:
             True: Full streaming support
             "no_tools": Streaming supported but not with tools
@@ -565,6 +569,9 @@ class ChatBedrockConverse(BaseChatModel):
                 )
             )
             or
+            # OpenAI gpt-oss models
+            (provider == "openai" and "gpt-oss" in model_id_lower)
+            or
             # Cohere Command R models
             (provider == "cohere" and "command-r" in model_id_lower)
         ):
@@ -609,7 +616,7 @@ class ChatBedrockConverse(BaseChatModel):
     @classmethod
     def set_disable_streaming(cls, values: Dict) -> Any:
         model_id = values.get("model_id", values.get("model"))
-        
+
         # Extract provider from the model_id
         # (e.g., "amazon", "anthropic", "ai21", "meta", "mistral")
         if "provider" not in values or values["provider"] == "":
@@ -649,8 +656,8 @@ class ChatBedrockConverse(BaseChatModel):
     @model_validator(mode="after")
     def validate_environment(self) -> Self:
         """Validate that AWS credentials to and python package exists in environment."""
-        
-         # Create bedrock client for control plane API call
+
+        # Create bedrock client for control plane API call
         if self.bedrock_client is None:
             self.bedrock_client = create_aws_client(
                 region_name=self.region_name,
@@ -662,7 +669,7 @@ class ChatBedrockConverse(BaseChatModel):
                 config=self.config,
                 service_name="bedrock",
             )
-            
+
         # Handle streaming configuration for application inference profiles
         if "application-inference-profile" in self.model_id:
             self._configure_streaming_for_resolved_model()
@@ -671,21 +678,34 @@ class ChatBedrockConverse(BaseChatModel):
         # only claude-3/4, mistral-large, and nova models support tool choice:
         # https://docs.aws.amazon.com/bedrock/latest/APIReference/API_runtime_ToolChoice.html
         if self.supports_tool_choice_values is None:
-            if "claude" in self._get_base_model():
+            base_model = self._get_base_model()
+            if "claude" in base_model:
                 # Tool choice not supported when thinking is enabled
                 thinking_params = (self.additional_model_request_fields or {}).get(
                     "thinking", {}
                 )
                 if (
-                    "claude-3-7-sonnet" in self._get_base_model()
+                    "claude-3-7-sonnet" in base_model
                     and thinking_params.get("type") == "enabled"
                 ):
                     self.supports_tool_choice_values = ()
                 else:
                     self.supports_tool_choice_values = ("auto", "any", "tool")
-            elif "mistral-large" in self._get_base_model():
+            elif "llama4" in base_model:
+                self.supports_tool_choice_values = ("auto",)
+            elif "llama3" in base_model:
+                if any(x in base_model for x in ("llama3-1", "llama3-3")):
+                    self.supports_tool_choice_values = ("auto",)
+                elif "llama3-2" in base_model:
+                    if any(x in base_model for x in ("11b", "90b")):
+                        self.supports_tool_choice_values = ("auto",)
+                    else:
+                        self.supports_tool_choice_values = ()
+                else:
+                    self.supports_tool_choice_values = ()
+            elif "mistral-large" in base_model:
                 self.supports_tool_choice_values = ("auto", "any")
-            elif "nova" in self._get_base_model():
+            elif "nova" in base_model:
                 self.supports_tool_choice_values = ("auto", "any", "tool")
             else:
                 self.supports_tool_choice_values = ()
@@ -709,27 +729,30 @@ class ChatBedrockConverse(BaseChatModel):
                 "Provide a guardrail via `guardrail_config` or "
                 "disable `guard_last_turn_only`."
             )
-            
+
         return self
 
     def _get_base_model(self) -> str:
         # identify the base model id used in the application inference profile (AIP)
         # Format: arn:aws:bedrock:us-east-1:<accountId>:application-inference-profile/<id>
-        if self.base_model_id is None and 'application-inference-profile' in self.model_id:
+        if (
+            self.base_model_id is None
+            and "application-inference-profile" in self.model_id
+        ):
             response = self.bedrock_client.get_inference_profile(
                 inferenceProfileIdentifier=self.model_id
             )
-            if 'models' in response and len(response['models']) > 0:
-                model_arn = response['models'][0]['modelArn']
+            if "models" in response and len(response["models"]) > 0:
+                model_arn = response["models"][0]["modelArn"]
                 # Format: arn:aws:bedrock:region::foundation-model/provider.model-name
-                self.base_model_id = model_arn.split('/')[-1]
+                self.base_model_id = model_arn.split("/")[-1]
         return self.base_model_id if self.base_model_id else self.model_id
-        
+
     def _configure_streaming_for_resolved_model(self) -> None:
         """Configure streaming support after resolving the base model for application inference profiles."""
         base_model = self._get_base_model()
         model_id_lower = base_model.lower()
-        
+
         streaming_support = self._get_streaming_support(self.provider, model_id_lower)
 
         # Set the disable_streaming flag accordingly
@@ -780,6 +803,21 @@ class ChatBedrockConverse(BaseChatModel):
                 kwargs, excluded_keys={"inputSchema", "properties", "thinking"}
             ),
         )
+
+        # Check for tool blocks without toolConfig and handle conversion
+        if params.get("toolConfig") is None and _has_tool_use_or_result_blocks(bedrock_messages):
+            logger.warning(
+                "Tool messages (toolUse/toolResult) detected without toolConfig. "
+                "Converting tool blocks to text format to avoid ValidationException."
+            )
+            warnings.warn(
+                "Tool messages were passed without toolConfig, converting to text format",
+                RuntimeWarning,
+            )
+
+            bedrock_messages = _convert_tool_blocks_to_text(bedrock_messages)
+            logger.debug(f"converted input messages: {bedrock_messages}")
+
         logger.debug(f"Input params: {params}")
         logger.info("Using Bedrock Converse API to generate response")
         response = self.client.converse(
@@ -812,6 +850,21 @@ class ChatBedrockConverse(BaseChatModel):
                 kwargs, excluded_keys={"inputSchema", "properties", "thinking"}
             ),
         )
+
+        # Check for tool blocks without toolConfig and handle conversion
+        if params.get("toolConfig") is None and _has_tool_use_or_result_blocks(bedrock_messages):
+            logger.warning(
+                "Tool messages (toolUse/toolResult) detected without toolConfig. "
+                "Converting tool blocks to text format to avoid ValidationException."
+            )
+            warnings.warn(
+                "Tool messages were passed without toolConfig, converting to text format",
+                RuntimeWarning,
+            )
+
+            bedrock_messages = _convert_tool_blocks_to_text(bedrock_messages)
+            logger.debug(f"converted input messages: {bedrock_messages}")
+
         response = self.client.converse_stream(
             messages=bedrock_messages, system=system, **params
         )
@@ -1114,6 +1167,10 @@ def _messages_to_bedrock(
             bedrock_messages.append(curr)
         else:
             raise ValueError(f"Unsupported message type {type(msg)}")
+        
+    if not bedrock_messages:
+        bedrock_messages.append({"role": "user", "content": [{"text": EMPTY_CONTENT}]})
+
     return bedrock_messages, bedrock_system
 
 
@@ -1189,7 +1246,10 @@ def _parse_stream_event(event: Dict[str, Any]) -> Optional[BaseMessageChunk]:
                     index=event["contentBlockStart"]["contentBlockIndex"],
                 )
             )
-        return AIMessageChunk(content=[block], tool_call_chunks=tool_call_chunks)
+        # always keep block inside a list to preserve merging compatibility
+        content = [block]
+
+        return AIMessageChunk(content=content, tool_call_chunks=tool_call_chunks)
     elif "contentBlockDelta" in event:
         block = {
             **_bedrock_to_lc([event["contentBlockDelta"]["delta"]])[0],
@@ -1205,19 +1265,20 @@ def _parse_stream_event(event: Dict[str, Any]) -> Optional[BaseMessageChunk]:
                     index=event["contentBlockDelta"]["contentBlockIndex"],
                 )
             )
-        return AIMessageChunk(content=[block], tool_call_chunks=tool_call_chunks)
+        # always keep block inside a list to preserve merging compatibility
+        content = [block]
+
+        return AIMessageChunk(content=content, tool_call_chunks=tool_call_chunks)
     elif "contentBlockStop" in event:
         # TODO: needed?
-        return AIMessageChunk(
-            content=[{"index": event["contentBlockStop"]["contentBlockIndex"]}]
-        )
+        return AIMessageChunk(content=[])
     elif "messageStop" in event:
         # TODO: snake case response metadata?
-        return AIMessageChunk(content=[], response_metadata=event["messageStop"])
+        return AIMessageChunk(content="", response_metadata=event["messageStop"])
     elif "metadata" in event:
         usage = _extract_usage_metadata(event["metadata"])
         return AIMessageChunk(
-            content=[], response_metadata=event["metadata"], usage_metadata=usage
+            content="", response_metadata=event["metadata"], usage_metadata=usage
         )
     elif "Exception" in list(event.keys())[0]:
         name, info = list(event.items())[0]
@@ -1237,13 +1298,13 @@ def _mime_type_to_format(mime_type: str) -> str:
 
     if mime_type in MIME_TO_FORMAT:
         return MIME_TO_FORMAT[mime_type]
-    
+
     # Fallback to original method of splitting on "/" for simple cases
     all_formats = set(MIME_TO_FORMAT.values())
     format_part = mime_type.split("/")[1]
     if format_part in all_formats:
         return format_part
-    
+
     raise ValueError(
         f"Unsupported MIME type: {mime_type}. Please refer to the Bedrock Converse API documentation for supported formats."
     )
@@ -1277,6 +1338,8 @@ def _format_data_content_block(block: dict) -> dict:
                     "source": {"bytes": _b64str_to_bytes(block["data"])},
                 }
             }
+            if citations := block.get("citations"):
+                formatted_block["document"]["citations"] = citations
             if name := block.get("name"):
                 formatted_block["document"]["name"] = name
             elif name := block.get("filename"):  # OpenAI uses `filename`
@@ -1301,7 +1364,13 @@ def _lc_content_to_bedrock(
     content: Union[str, List[Union[str, Dict[str, Any]]]],
 ) -> List[Dict[str, Any]]:
     if isinstance(content, str):
-        content = [{"text": content}]
+        if not content or content.isspace():
+            content = [{"text": EMPTY_CONTENT}]
+        else:
+            content = [{"text": content}]
+    elif isinstance(content, list) and len(content) == 0:
+        content = [{"type": "text", "text": EMPTY_CONTENT}]
+
     bedrock_content: List[Dict[str, Any]] = []
     for block in _snake_to_camel_keys(content):
         if isinstance(block, str):
@@ -1314,7 +1383,27 @@ def _lc_content_to_bedrock(
         ):
             bedrock_content.append(_format_data_content_block(block))
         elif block["type"] == "text":
-            bedrock_content.append({"text": block["text"]})
+            if not block["text"] or (isinstance(block["text"], str) and block["text"].isspace()):
+                bedrock_content.append({"text": EMPTY_CONTENT})
+            else:
+                text_block = {"text": block["text"]}
+                if (
+                    (citations := block.get("citations"))
+                    and isinstance(citations, list)
+                    and len(citations) > 0
+                    and isinstance(citations[0], dict)
+                    and "sourceContent" in citations[0]  # validate format
+                ):
+                    bedrock_content.append(
+                        {
+                            "citationsContent": {
+                                "content": [text_block],
+                                "citations": citations,
+                            }
+                        }
+                    )
+                else:
+                    bedrock_content.append(text_block)
         elif block["type"] == "image":
             # Assume block is already in bedrock format.
             if "image" in block:
@@ -1323,7 +1412,9 @@ def _lc_content_to_bedrock(
                 bedrock_content.append(
                     {
                         "image": {
-                            "format": _mime_type_to_format(block["source"]["mediaType"]),
+                            "format": _mime_type_to_format(
+                                block["source"]["mediaType"]
+                            ),
                             "source": {
                                 "bytes": _b64str_to_bytes(block["source"]["data"])
                             },
@@ -1344,7 +1435,9 @@ def _lc_content_to_bedrock(
                     bedrock_content.append(
                         {
                             "video": {
-                                "format": _mime_type_to_format(block["source"]["mediaType"]),
+                                "format": _mime_type_to_format(
+                                    block["source"]["mediaType"]
+                                ),
                                 "source": {
                                     "bytes": _b64str_to_bytes(block["source"]["data"])
                                 },
@@ -1355,7 +1448,9 @@ def _lc_content_to_bedrock(
                     bedrock_content.append(
                         {
                             "video": {
-                                "format": _mime_type_to_format(block["source"]["mediaType"]),
+                                "format": _mime_type_to_format(
+                                    block["source"]["mediaType"]
+                                ),
                                 "source": {"s3Location": block["source"]["data"]},
                             }
                         }
@@ -1544,6 +1639,15 @@ def _bedrock_to_lc(content: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
                         text_block["citations"] = citations
                     lc_content.append(text_block)
 
+        elif "citation" in block:  # streaming citations
+            lc_content.append(
+                {
+                    "type": "text",
+                    "text": "",
+                    "citations": [block["citation"]]
+                }
+            )
+
         else:
             raise ValueError(
                 "Unexpected content block type in content. Expected to have one of "
@@ -1675,6 +1779,8 @@ def _str_if_single_text_block(
 def _upsert_tool_calls_to_bedrock_content(
     content: List[Dict[str, Any]], tool_calls: List[ToolCall]
 ) -> List[Dict[str, Any]]:
+    if tool_calls and content == [{"text": EMPTY_CONTENT}]:
+        content = []
     existing_tc_blocks = [block for block in content if "toolUse" in block]
     for tool_call in tool_calls:
         if tool_call["id"] in [
@@ -1746,3 +1852,62 @@ def _is_cache_point(cache_point: Any) -> bool:
         and "cachePoint" in cache_point
         and cache_point.get("cachePoint").get("type") is not None
     )
+
+
+def _has_tool_use_or_result_blocks(messages: List[Dict[str, Any]]) -> bool:
+    """Check if messages contain toolUse or toolResult blocks."""
+    for message in messages:
+        for block in message.get("content", []):
+            if "toolUse" in block or "toolResult" in block:
+                return True
+    return False
+
+
+def _convert_tool_blocks_to_text(
+    messages: List[Dict[str, Any]],
+) -> List[Dict[str, Any]]:
+    """Convert toolUse and toolResult blocks to text blocks preserving only necessary content."""
+    converted_messages = []
+
+    for message in messages:
+        converted_message = {"role": message["role"], "content": []}
+
+        for block in message.get("content", []):
+            if "toolUse" in block:
+                # convert toolUse to simple text
+                tool_use = block["toolUse"]
+                tool_name = tool_use.get("name", "function")
+                tool_inputs = tool_use.get("input", {})
+
+                # format function call description
+                if tool_inputs:
+                    tool_text = f"[Called {tool_name} with parameters: {json.dumps(tool_inputs)}]"
+                else:
+                    tool_text = f"[Called {tool_name}]"
+
+                converted_message["content"].append({"text": tool_text})
+
+            elif "toolResult" in block:
+                # convert toolResult to indicate it's tool output without exposing internal details
+                tool_result = block["toolResult"]
+
+                content_parts = []
+                for content_block in tool_result.get("content", []):
+                    if "text" in content_block:
+                        content_parts.append(content_block["text"])
+                    elif "json" in content_block:
+                        content_parts.append(json.dumps(content_block["json"]))
+                    # skip other internal content types
+                result_content = "".join(content_parts)
+
+                # only include result if there's actual content, but mark it as tool output
+                if result_content.strip():
+                    tool_output_text = f"[Tool output: {result_content}]"
+                    converted_message["content"].append({"text": tool_output_text})
+            else:
+                # keep other blocks as they are
+                converted_message["content"].append(block)
+
+        converted_messages.append(converted_message)
+
+    return converted_messages
